@@ -3,13 +3,13 @@
 ###
 ### oktest.py -- new style test utility
 ###
-### $Release: 0.10.0 $
+### $Release: 0.11.0 $
 ### $Copyright: copyright(c) 2010-2011 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
 
 __all__ = ('ok', 'NOT', 'NG', 'not_ok', 'run', 'spec', 'test', 'fail', 'skip', 'todo', 'subject', 'situation', 'main')
-__version__ = "$Release: 0.10.0 $".split()[1]
+__version__ = "$Release: 0.11.0 $".split()[1]
 
 import sys, os, re, types, traceback, time, linecache
 
@@ -17,30 +17,10 @@ python2 = sys.version_info[0] == 2
 python3 = sys.version_info[0] == 3
 if python2:
     from cStringIO import StringIO
-    def _is_string(val):
-        return isinstance(val, (str, unicode))
-    def _is_class(obj):
-        return isinstance(obj, (types.TypeType, types.ClassType))
-    def _is_unbound(method):
-        return not method.im_self
-    def _func_name(func):
-        return func.func_name
-    def _func_firstlineno(func):
-        func = getattr(func, 'im_func', func)
-        return func.func_code.co_firstlineno
 if python3:
     xrange = range
     from io import StringIO
-    def _is_string(val):
-        return isinstance(val, (str, bytes))
-    def _is_class(obj):
-        return isinstance(obj, (type, ))
-    def _is_unbound(method):
-        return not method.__self__
-    def _func_name(func):
-        return func.__name__
-    def _func_firstlineno(func):
-        return func.__code__.co_firstlineno
+
 
 def _new_module(name, local_vars, util=None):
     mod = type(sys)(name)
@@ -62,6 +42,7 @@ config = _new_module('oktest.config', {
     #"color_enabled": any(lambda x: _sys.platform.startswith(x), ('darwin', 'linux', 'freebsd', 'netbsd'))  # not work on Python2.4
     "color_available": bool([ 1 for p in ('darwin', 'linux', 'freebsd', 'netbsd') if sys.platform.startswith(p) ]),
     "color_enabled":  None,    # None means detect automatiallly
+    "TARGET_PATTERN": '.*(Test|TestCase|_TC)$',   # class name pattern of test case
 })
 
 
@@ -78,19 +59,11 @@ config = _new_module('oktest.config', {
 ASSERTION_ERROR = AssertionError
 
 
-def ex2msg(ex):
-    #return ex.message   # deprecated since Python 2.6
-    #return str(ex)      # may be empty
-    #return ex.args[0]   # ex.args may be empty (ex. AssertionError)
-    #return (ex.args or ['(no error message)'])[0]
-    return str(ex) or '(no error message)'
-
-
 def _diff_p(target, op, other):
     if op != '==':             return False
     if target == other:        return False
-    #if not _is_string(target): return False
-    #if not _is_string(other):  return False
+    #if not util._is_string(target): return False
+    #if not util._is_string(other):  return False
     if not DIFF:               return False
     is_a = isinstance
     if is_a(target, str) and is_a(other, str):
@@ -159,8 +132,6 @@ def assertion(func):
     def deco(self, *args):
         self._tested = True
         return func(self, *args)
-    if python2:
-        deco.func_name = func.func_name
     deco.__name__ = func.__name__
     deco.__doc__ = func.__doc__
     setattr(AssertionObject, func.__name__, deco)
@@ -363,13 +334,14 @@ def _f():
             self.failed("re.search(%r, %r) : failed." % (pattern, self.target))
 
     @assertion
-    def not_match(self, pattern):  # DEPRECATED
+    def not_match(self, pattern, flag=0):
         if isinstance(pattern, type(re.compile('x'))):
             boolean = not pattern.search(self.target)
             if boolean == self.boolean:  return self
             self.failed("not re.search(%r, %r) : failed." % (pattern.pattern, self.target))
         else:
-            boolean = not re.search(pattern, self.target)
+            rexp = re.compile(pattern, flag)
+            boolean = not rexp.search(self.target)
             if boolean == self.boolean:  return self
             self.failed("not re.search(%r, %r) : failed." % (pattern, self.target))
 
@@ -386,7 +358,7 @@ def _f():
         self.failed('os.path.isfile(%r) : failed.' % self.target)
 
     @assertion
-    def is_not_file(self):  # DEPRECATED
+    def not_file(self):
         boolean = not os.path.isfile(self.target)
         if boolean == self.boolean:  return self
         self.failed('not os.path.isfile(%r) : failed.' % self.target)
@@ -398,7 +370,7 @@ def _f():
         self.failed('os.path.isdir(%r) : failed.' % self.target)
 
     @assertion
-    def is_not_dir(self):  # DEPRECATED
+    def not_dir(self):
         boolean = not os.path.isdir(self.target)
         if boolean == self.boolean:  return self
         self.failed('not os.path.isdir(%r) : failed.' % self.target)
@@ -454,7 +426,9 @@ def _f():
         return self
 
     AssertionObject._raise_or_not = _raise_or_not
-    AssertionObject.hasattr = has_attr    # for backward compatibility
+    AssertionObject.hasattr = has_attr      # for backward compatibility
+    AssertionObject.is_not_file = not_file  # for backward compatibility
+    AssertionObject.is_not_dir  = not_dir   # for backward compatibility
 
 _f()
 del _f
@@ -525,10 +499,11 @@ class Should(object):
 class SkipTest(Exception):
     pass
 
-import unittest as _unittest
-if hasattr(_unittest, 'SkipTest'):
-    SkipTest = _unittest.SkipTest
-del _unittest
+try:
+    from unittest import SkipTest
+except ImportError:
+    if python2:
+        sys.exc_clear()
 
 
 class SkipObject(object):
@@ -543,7 +518,7 @@ class SkipObject(object):
                     raise SkipTest(reason)
                 fn.__name__ = func.__name__
                 fn.__doc__  = func.__doc__
-                fn._firstlineno = _func_firstlineno(func)
+                fn._firstlineno = util._func_firstlineno(func)
                 return fn
         else:
             def deco(func):
@@ -572,6 +547,7 @@ def todo(func):
 class _ExpectedFailure(Exception):
 
     def __init__(self, exc_info=None):
+        Exception.__init__(self, "expected failure")
         if exc_info:
             self.exc_info = exc_info
 
@@ -581,7 +557,8 @@ class _UnexpectedSuccess(Exception):
 try:
     from unittest.case import _ExpectedFailure, _UnexpectedSuccess
 except ImportError:
-    pass
+    if python2:
+        sys.exc_clear()
 
 
 
@@ -639,7 +616,7 @@ class TestRunner(object):
         ## sort by linenumber
         def fn(testname, klass=klass):
             func = getattr(klass, testname)
-            lineno = getattr(func, '_firstlineno', None) or _func_firstlineno(func)
+            lineno = getattr(func, '_firstlineno', None) or util._func_firstlineno(func)
             return (lineno, testname)
         testnames.sort(key=fn)
         return testnames
@@ -790,9 +767,6 @@ class TestRunner(object):
         self.reporter.exit_all()
 
 
-OUT = None    # None means sys.stdout
-
-
 def _filtered(klass, meth, tname, pattern, key, val, _rexp=re.compile(r'^test(_|_\d\d\d(_|: ))?')):
     from fnmatch import fnmatch
     if pattern:
@@ -809,9 +783,6 @@ def _filtered(klass, meth, tname, pattern, key, val, _rexp=re.compile(r'^test(_|
 TEST_RUNNER = TestRunner
 
 
-TARGET_PATTERN = '.*(Test|TestCase|_TC)$'
-
-
 def run(*targets, **kwargs):
     out    = kwargs.pop('out', None)
     color  = kwargs.pop('color', None)
@@ -831,7 +802,7 @@ def run(*targets, **kwargs):
     runner = TEST_RUNNER(reporter=reporter, filter=filter)
     #
     if len(targets) == 0:
-        targets = (TARGET_PATTERN, )
+        targets = (config.TARGET_PATTERN, )
     #
     runner.__enter__()
     try:
@@ -850,13 +821,13 @@ def _target_classes(targets):
     rexp_type = type(re.compile('x'))
     vars = None
     for arg in targets:
-        if _is_class(arg):
+        if util._is_class(arg):
             klass = arg
             target_classes.append(klass)
-        elif _is_string(arg) or isinstance(arg, rexp_type):
-            rexp = _is_string(arg) and re.compile(arg) or arg
+        elif util._is_string(arg) or isinstance(arg, rexp_type):
+            rexp = util._is_string(arg) and re.compile(arg) or arg
             if vars is None: vars = sys._getframe(2).f_locals
-            klasses = [ vars[k] for k in vars if rexp.search(k) and _is_class(vars[k]) ]
+            klasses = [ vars[k] for k in vars if rexp.search(k) and util._is_class(vars[k]) ]
             if TESTCLASS_SORT_KEY:
                 klasses.sort(key=TESTCLASS_SORT_KEY)
             target_classes.extend(klasses)
@@ -868,7 +839,7 @@ def _target_classes(targets):
 def _min_firstlineno_of_methods(klass):
     func_types = (types.FunctionType, types.MethodType)
     d = klass.__dict__
-    linenos = [ _func_firstlineno(d[k]) for k in d
+    linenos = [ util._func_firstlineno(d[k]) for k in d
                 if k.startswith('test') and type(d[k]) in func_types ]
     return linenos and min(linenos) or -1
 
@@ -922,7 +893,7 @@ class BaseReporter(Reporter):
 
     def __get_out(self):
         if not self._out:
-            self.out = OUT or sys.stdout
+            self.out = sys.stdout
         return self._out
 
     def __set_out(self, out):
@@ -1121,13 +1092,11 @@ class BaseReporter(Reporter):
     def _filter_stacktrace(self, stacktrace, traceback_):
         entries = traceback.extract_tb(traceback_)
         file, line, func, text = entries[0]
-        def is_oktest_py(file):
-            return re.search(r'oktest.py[co]?$', file)
         i = len(stacktrace) - 1
         while i >= 0 and not (stacktrace[i][0] == file and stacktrace[i][2] == func):
             i -= 1
         bottom = i
-        while i >= 0 and not is_oktest_py(stacktrace[i][0]):
+        while i >= 0 and not _is_oktest_py(stacktrace[i][0]):
             i -= 1
         top = i + 1
         return stacktrace[top:bottom]
@@ -1140,16 +1109,16 @@ class BaseReporter(Reporter):
     def colorize(self, string, kind):
         if not self._color:
             return string
-        if kind == ST_PASSED:  return Color.green(string, bold=True)
-        if kind == ST_FAILED:  return Color.red(string, bold=True)
-        if kind == ST_ERROR:   return Color.red(string, bold=True)
-        if kind == ST_SKIPPED: return Color.yellow(string, bold=True)
-        if kind == ST_TODO:    return Color.yellow(string, bold=True)
-        #if kind == ST_UNEXPECTED: return Color.red(string, bold=True)
-        if kind == "topic":    return Color.bold(string)
-        if kind == "sep":      return Color.red(string)
-        if kind == "context":  return Color.bold(string)
-        return Color.yellow(string)
+        if kind == ST_PASSED:  return util.Color.green(string, bold=True)
+        if kind == ST_FAILED:  return util.Color.red(string, bold=True)
+        if kind == ST_ERROR:   return util.Color.red(string, bold=True)
+        if kind == ST_SKIPPED: return util.Color.yellow(string, bold=True)
+        if kind == ST_TODO:    return util.Color.yellow(string, bold=True)
+        #if kind == ST_UNEXPECTED: return util.Color.red(string, bold=True)
+        if kind == "topic":    return util.Color.bold(string)
+        if kind == "sep":      return util.Color.red(string)
+        if kind == "context":  return util.Color.bold(string)
+        return util.Color.yellow(string)
 
     def write_separator(self):
         self.out.write(self.colorize(self.separator, "sep") + "\n")
@@ -1178,6 +1147,11 @@ class BaseReporter(Reporter):
         return cls._registered.get(name)
 
 
+def _is_oktest_py(filepath, _dirpath=os.path.dirname(__file__)):
+    #return re.search(r'oktest.py[co]?$', filepath)
+    return filepath.startswith(_dirpath)
+
+
 def is_tty(out):
     return hasattr(out, 'isatty') and out.isatty()
 
@@ -1384,7 +1358,7 @@ class OldStyleReporter(BaseReporter):
         elif status == ST_FAILED:
             ex_class, ex, ex_traceback = exc_info
             flag = hasattr(ex, '_raised_by_oktest')
-            self.out.write("[NG] %s\n" % (flag and ex.errmsg or ex2msg(ex)))
+            self.out.write("[NG] %s\n" % (flag and ex.errmsg or util.ex2msg(ex)))
             def formatter(filepath, lineno, funcname, linestr):
                 return "   %s:%s: %s\n" % (filepath, lineno, linestr.strip())
             arr = format_traceback(ex, ex_traceback, filter=self._filter, formatter=formatter)
@@ -1394,7 +1368,7 @@ class OldStyleReporter(BaseReporter):
                 self.out.write(ex.diff)
         elif status == ST_ERROR:
             ex_class, ex, ex_traceback = exc_info
-            self.out.write("[ERROR] %s: %s\n" % (ex_class.__name__, ex2msg(ex)))
+            self.out.write("[ERROR] %s: %s\n" % (ex_class.__name__, util.ex2msg(ex)))
             def formatter(filepath, lineno, funcname, linestr):
                 return "  - %s:%s:  %s\n" % (filepath, lineno, linestr.strip())
             arr = format_traceback(ex, ex_traceback, filter=self._filter, formatter=formatter)
@@ -1423,88 +1397,265 @@ if os.environ.get('OKTEST_REPORTER'):
 
 
 ##
-## color
+## util
 ##
-class Color(object):
+def _dummy():
 
-    @staticmethod
-    def bold(s):
-        return "\033[0;1m" + s + "\033[22m"
+    __all__ = ('chdir', 'rm_rf')
 
-    @staticmethod
-    def black(s, bold=False):
-        return "\033[%s;30m%s\033[0m" % (bold and 1 or 0, s)
+    if python2:
+        def _is_string(val):
+            return isinstance(val, (str, unicode))
+        def _is_class(obj):
+            return isinstance(obj, (types.TypeType, types.ClassType))
+        def _is_unbound(method):
+            return not method.im_self
+        def _func_name(func):
+            return func.func_name
+        def _func_firstlineno(func):
+            func = getattr(func, 'im_func', func)
+            return func.func_code.co_firstlineno
+    if python3:
+        def _is_string(val):
+            return isinstance(val, (str, bytes))
+        def _is_class(obj):
+            return isinstance(obj, (type, ))
+        def _is_unbound(method):
+            return not method.__self__
+        def _func_name(func):
+            return func.__name__
+        def _func_firstlineno(func):
+            return func.__code__.co_firstlineno
 
-    @staticmethod
-    def red(s, bold=False):
-        return "\033[%s;31m%s\033[0m" % (bold and 1 or 0, s)
+    ##
+    ## _Context
+    ##
+    class Context(object):
 
-    @staticmethod
-    def green(s, bold=False):
-        return "\033[%s;32m%s\033[0m" % (bold and 1 or 0, s)
+        def __enter__(self):
+            return self
 
-    @staticmethod
-    def yellow(s, bold=False):
-        return "\033[%s;33m%s\033[0m" % (bold and 1 or 0, s)
-
-    @staticmethod
-    def blue(s, bold=False):
-        return "\033[%s;34m%s\033[0m" % (bold and 1 or 0, s)
-
-    @staticmethod
-    def magenta(s, bold=False):
-        return "\033[%s;35m%s\033[0m" % (bold and 1 or 0, s)
-
-    @staticmethod
-    def cyan(s, bold=False):
-        return "\033[%s;36m%s\033[0m" % (bold and 1 or 0, s)
-
-    @staticmethod
-    def white(s, bold=False):
-        return "\033[%s;37m%s\033[0m" % (bold and 1 or 0, s)
-
-    @staticmethod
-    def _colorize(s):
-        s = re.sub(r'<b>(.*?)</b>', lambda m: Color.bold(m.group(1)), s)
-        s = re.sub(r'<R>(.*?)</R>', lambda m: Color.red(m.group(1), bold=True), s)
-        s = re.sub(r'<r>(.*?)</r>', lambda m: Color.red(m.group(1), bold=False), s)
-        s = re.sub(r'<G>(.*?)</G>', lambda m: Color.green(m.group(1), bold=True), s)
-        s = re.sub(r'<Y>(.*?)</Y>', lambda m: Color.yellow(m.group(1), bold=True), s)
-        return s
-
-##
-## _Context
-##
-class _Context(object):
-
-    def __enter__(self):
-        return self
-
-    def __exit__(self, *args):
-        return None
+        def __exit__(self, *args):
+            return None
 
 
-class _RunnableContext(_Context):
+    class RunnableContext(Context):
 
-    def run(self, func, *args, **kwargs):
-        self.__enter__()
+        def run(self, func, *args, **kwargs):
+            self.__enter__()
+            try:
+                return func(*args, **kwargs)
+            finally:
+                self.__exit__(*sys.exc_info())
+
+        def deco(self, func):
+            def f(*args, **kwargs):
+                return self.run(func, *args, **kwargs)
+            return f
+
+        __call__ = run    # for backward compatibility
+
+
+    class Chdir(RunnableContext):
+
+        def __init__(self, dirname):
+            self.dirname = dirname
+            self.path    = os.path.abspath(dirname)
+            self.back_to = os.getcwd()
+
+        def __enter__(self, *args):
+            os.chdir(self.path)
+            return self
+
+        def __exit__(self, *args):
+            os.chdir(self.back_to)
+
+
+    class Using(Context):
+        """ex.
+             class MyTest(object):
+                pass
+             with oktest.util.Using(MyTest):
+                def test_1(self):
+                  ok (1+1) == 2
+             if __name__ == '__main__':
+                oktest.run(MyTest)
+        """
+        def __init__(self, klass):
+            self.klass = klass
+
+        def __enter__(self):
+            localvars = sys._getframe(1).f_locals
+            self._start_names = localvars.keys()
+            if python3: self._start_names = list(self._start_names)
+            return self
+
+        def __exit__(self, *args):
+            localvars  = sys._getframe(1).f_locals
+            curr_names = localvars.keys()
+            diff_names = list(set(curr_names) - set(self._start_names))
+            for name in diff_names:
+                setattr(self.klass, name, localvars[name])
+
+
+    def chdir(path, func=None):
+        cd = Chdir(path)
+        return func is not None and cd.run(func) or cd
+
+    def using(klass):                       ## undocumented
+        return Using(klass)
+
+
+    def ex2msg(ex):
+        #return ex.message   # deprecated since Python 2.6
+        #return str(ex)      # may be empty
+        #return ex.args[0]   # ex.args may be empty (ex. AssertionError)
+        #return (ex.args or ['(no error message)'])[0]
+        return str(ex) or '(no error message)'
+
+    def flatten(arr, type=(list, tuple)):   ## undocumented
+        L = []
+        for x in arr:
+            if isinstance(x, type):
+                L.extend(flatten(x))
+            else:
+                L.append(x)
+        return L
+
+    def rm_rf(*fnames):
+        for fname in flatten(fnames):
+            if os.path.isfile(fname):
+                os.unlink(fname)
+            elif os.path.isdir(fname):
+                from shutil import rmtree
+                rmtree(fname)
+
+    def get_location(depth=0):
+        frame = sys._getframe(depth+1)
+        return (frame.f_code.co_filename, frame.f_lineno)
+
+    def read_binary_file(fname):
+        f = open(fname, 'rb')
         try:
-            return func(*args, **kwargs)
+            b = f.read()
         finally:
-            self.__exit__(*sys.exc_info())
+            f.close()
+        return b
 
-    def deco(self, func):
-        def f(*args, **kwargs):
-            return self.run(func, *args, **kwargs)
-        return f
+    if python2:
+        _rexp = re.compile(r'(?:^#!.*?\r?\n)?#.*?coding:[ \t]*([-\w]+)')
+        def read_text_file(fname,  _rexp=_rexp, _read_binary_file=read_binary_file):
+            b = _read_binary_file(fname)
+            m = _rexp.match(b)
+            encoding = m and m.group(1) or 'utf-8'
+            u = b.decode(encoding)
+            assert isinstance(u, unicode)
+            return u
+    if python3:
+        _rexp = re.compile(r'(?:^#!.*?\r?\n)?#.*?coding:[ \t]*([-\w]+)'.encode('us-ascii'))
+        def read_text_file(fname,  _rexp=_rexp, _read_binary_file=read_binary_file):
+            b = _read_binary_file(fname)
+            m = _rexp.match(b)
+            encoding = m and m.group(1).decode('us-ascii') or 'utf-8'
+            u = b.decode(encoding)
+            assert isinstance(u, str)
+            return u
 
-    __call__ = run    # for backward compatibility
+    from types import MethodType as _MethodType
+
+    if python2:
+        def func_argnames(func):
+            if isinstance(func, _MethodType):
+                codeobj = func.im_func.func_code
+                index = 1
+            else:
+                codeobj = func.func_code
+                index = 0
+            return codeobj.co_varnames[index:codeobj.co_argcount]
+        def func_defaults(func):
+            if isinstance(func, _MethodType):
+                return func.im_func.func_defaults
+            else:
+                return func.func_defaults
+    if python3:
+        def func_argnames(func):
+            if isinstance(func, _MethodType):
+                codeobj = func.__func__.__code__
+                index = 1
+            else:
+                codeobj = func.__code__
+                index = 0
+            return codeobj.co_varnames[index:codeobj.co_argcount]
+        def func_defaults(func):
+            if isinstance(func, _MethodType):
+                return func.__func__.__defaults__
+            else:
+                return func.__defaults__
+
+    ##
+    ## color
+    ##
+    class Color(object):
+
+        @staticmethod
+        def bold(s):
+            return "\033[0;1m" + s + "\033[22m"
+
+        @staticmethod
+        def black(s, bold=False):
+            return "\033[%s;30m%s\033[0m" % (bold and 1 or 0, s)
+
+        @staticmethod
+        def red(s, bold=False):
+            return "\033[%s;31m%s\033[0m" % (bold and 1 or 0, s)
+
+        @staticmethod
+        def green(s, bold=False):
+            return "\033[%s;32m%s\033[0m" % (bold and 1 or 0, s)
+
+        @staticmethod
+        def yellow(s, bold=False):
+            return "\033[%s;33m%s\033[0m" % (bold and 1 or 0, s)
+
+        @staticmethod
+        def blue(s, bold=False):
+            return "\033[%s;34m%s\033[0m" % (bold and 1 or 0, s)
+
+        @staticmethod
+        def magenta(s, bold=False):
+            return "\033[%s;35m%s\033[0m" % (bold and 1 or 0, s)
+
+        @staticmethod
+        def cyan(s, bold=False):
+            return "\033[%s;36m%s\033[0m" % (bold and 1 or 0, s)
+
+        @staticmethod
+        def white(s, bold=False):
+            return "\033[%s;37m%s\033[0m" % (bold and 1 or 0, s)
+
+        @staticmethod
+        def _colorize(s):
+            s = re.sub(r'<b>(.*?)</b>', lambda m: Color.bold(m.group(1)), s)
+            s = re.sub(r'<R>(.*?)</R>', lambda m: Color.red(m.group(1), bold=True), s)
+            s = re.sub(r'<r>(.*?)</r>', lambda m: Color.red(m.group(1), bold=False), s)
+            s = re.sub(r'<G>(.*?)</G>', lambda m: Color.green(m.group(1), bold=True), s)
+            s = re.sub(r'<Y>(.*?)</Y>', lambda m: Color.yellow(m.group(1), bold=True), s)
+            return s
+
+
+    return locals()
+
+util = _new_module('oktest.util', _dummy())
+del _dummy
+
+helper = util  ## 'help' is an alias of 'util' (for backward compatibility)
+sys.modules['oktest.helper'] = sys.modules['oktest.util']
 
 
 ##
 ## spec()   # deprecated
 ##
-class Spec(_Context):   # deprecated
+class Spec(util.Context):   # deprecated
 
     _exception  = None
     _traceback  = None
@@ -1559,6 +1710,13 @@ class Spec(_Context):   # deprecated
         if ex:
             raise ex
 
+    def __call__(self, func):
+        self.__enter__()
+        try:
+            func()
+        finally:
+            self.__exit__(*sys.exc_info())
+
     def __bool__(self):       # for Pyton3
         filter = os.environ.get('SPEC')
         return not filter or (filter in self.desc)
@@ -1567,9 +1725,9 @@ class Spec(_Context):   # deprecated
 
 
 def spec(desc):   # deprecated
-    if not os.getenv('OKTEST_WARNING_DISABLED'):
-        import warnings
-        warnings.warn("oktest.spec() is deprecated.", DeprecationWarning, 2)
+    #if not os.getenv('OKTEST_WARNING_DISABLED'):
+    #    import warnings
+    #    warnings.warn("oktest.spec() is deprecated.", DeprecationWarning, 2)
     return Spec(desc)
 
 
@@ -1604,7 +1762,7 @@ def test(description_text=None, **options):
             localvars[newfunc.__name__] = newfunc
         newfunc.__doc__  = orig_func.__doc__ or description_text
         newfunc._options = options
-        newfunc._firstlineno = getattr(orig_func, '_firstlineno', None) or _func_firstlineno(orig_func)
+        newfunc._firstlineno = getattr(orig_func, '_firstlineno', None) or util._func_firstlineno(orig_func)
         return newfunc
     return deco
 
@@ -1686,7 +1844,11 @@ class FixtureInjector(object):
             if name in releasers:
                 releaser = releasers[name]
                 if releaser:
-                    releaser(resolved[name])
+                    names = util.func_argnames(releaser)
+                    if names and names[0] == "self":
+                        releaser(resolved["self"], resolved[name])
+                    else:
+                        releaser(resolved[name])
             else:
                 fixture_manager.release(name, resolved[name])
 
@@ -1808,7 +1970,7 @@ def context():
             def fn(item):
                 if isinstance(item, tuple):
                     return getattr(item[1], '_firstlineno', None) or \
-                           _func_firstlineno(item[1])
+                           util._func_firstlineno(item[1])
                 elif isinstance(item, TestContext):
                     return item._lineno or 0
                 else:
@@ -1848,154 +2010,6 @@ subject   = context.subject
 situation = context.situation
 
 
-##
-## util
-##
-def _dummy():
-
-    __all__ = ('chdir', 'rm_rf')
-
-
-    class Chdir(_RunnableContext):
-
-        def __init__(self, dirname):
-            self.dirname = dirname
-            self.path    = os.path.abspath(dirname)
-            self.back_to = os.getcwd()
-
-        def __enter__(self, *args):
-            os.chdir(self.path)
-            return self
-
-        def __exit__(self, *args):
-            os.chdir(self.back_to)
-
-
-    class Using(_Context):
-        """ex.
-             class MyTest(object):
-                pass
-             with oktest.Using(MyTest):
-                def test_1(self):
-                  ok (1+1) == 2
-             if __name__ == '__main__':
-                oktest.run(MyTest)
-        """
-        def __init__(self, klass):
-            self.klass = klass
-
-        def __enter__(self):
-            localvars = sys._getframe(1).f_locals
-            self._start_names = localvars.keys()
-            if python3: self._start_names = list(self._start_names)
-            return self
-
-        def __exit__(self, *args):
-            localvars  = sys._getframe(1).f_locals
-            curr_names = localvars.keys()
-            diff_names = list(set(curr_names) - set(self._start_names))
-            for name in diff_names:
-                setattr(self.klass, name, localvars[name])
-
-
-    def chdir(path, func=None):
-        cd = Chdir(path)
-        return func is not None and cd.run(func) or cd
-
-    def using(klass):                       ## undocumented
-        return Using(klass)
-
-
-    def flatten(arr, type=(list, tuple)):   ## undocumented
-        L = []
-        for x in arr:
-            if isinstance(x, type):
-                L.extend(flatten(x))
-            else:
-                L.append(x)
-        return L
-
-    def rm_rf(*fnames):
-        for fname in flatten(fnames):
-            if os.path.isfile(fname):
-                os.unlink(fname)
-            elif os.path.isdir(fname):
-                from shutil import rmtree
-                rmtree(fname)
-
-    def get_location(depth=0):
-        frame = sys._getframe(depth+1)
-        return (frame.f_code.co_filename, frame.f_lineno)
-
-    def read_binary_file(fname):
-        f = open(fname, 'rb')
-        try:
-            b = f.read()
-        finally:
-            f.close()
-        return b
-
-    if python2:
-        _rexp = re.compile(r'(?:^#!.*?\r?\n)?#.*?coding:[ \t]*([-\w]+)')
-        def read_text_file(fname,  _rexp=_rexp, _read_binary_file=read_binary_file):
-            b = _read_binary_file(fname)
-            m = _rexp.match(b)
-            encoding = m and m.group(1) or 'utf-8'
-            u = b.decode(encoding)
-            assert isinstance(u, unicode)
-            return u
-    if python3:
-        _rexp = re.compile(r'(?:^#!.*?\r?\n)?#.*?coding:[ \t]*([-\w]+)'.encode('us-ascii'))
-        def read_text_file(fname,  _rexp=_rexp, _read_binary_file=read_binary_file):
-            b = _read_binary_file(fname)
-            m = _rexp.match(b)
-            encoding = m and m.group(1).decode('us-ascii') or 'utf-8'
-            u = b.decode(encoding)
-            assert isinstance(u, str)
-            return u
-
-    from types import MethodType as _MethodType
-
-    if python2:
-        def func_argnames(func):
-            if isinstance(func, _MethodType):
-                codeobj = func.im_func.func_code
-                index = 1
-            else:
-                codeobj = func.func_code
-                index = 0
-            return codeobj.co_varnames[index:codeobj.co_argcount]
-        def func_defaults(func):
-            if isinstance(func, _MethodType):
-                return func.im_func.func_defaults
-            else:
-                return func.func_defaults
-    if python3:
-        def func_argnames(func):
-            if isinstance(func, _MethodType):
-                codeobj = func.__func__.__code__
-                index = 1
-            else:
-                codeobj = func.__code__
-                index = 0
-            return codeobj.co_varnames[index:codeobj.co_argcount]
-        def func_defaults(func):
-            if isinstance(func, _MethodType):
-                return func.__func__.__defaults__
-            else:
-                return func.__defaults__
-
-    return locals()
-
-
-util = _new_module('oktest.util', _dummy())
-del _dummy
-
-helper = util  ## 'help' is an alias of 'util' (for backward compatibility)
-sys.modules['oktest.helper'] = sys.modules['oktest.util']
-
-
-
 ##
 ## dummy
 ##
@@ -2004,7 +2018,7 @@ def _dummy():
     __all__ = ('dummy_file', 'dummy_dir', 'dummy_values', 'dummy_attrs', 'dummy_environ_vars', 'dummy_io')
 
 
-    class DummyFile(_RunnableContext):
+    class DummyFile(util.RunnableContext):
 
         def __init__(self, filename, content):
             self.filename = filename
@@ -2023,7 +2037,7 @@ def _dummy():
             os.unlink(self.path)
 
 
-    class DummyDir(_RunnableContext):
+    class DummyDir(util.RunnableContext):
 
         def __init__(self, dirname):
             self.dirname = dirname
@@ -2038,7 +2052,7 @@ def _dummy():
             shutil.rmtree(self.path)
 
 
-    class DummyValues(_RunnableContext):
+    class DummyValues(util.RunnableContext):
 
         def __init__(self, dictionary, items_=None, **kwargs):
             self.dict = dictionary
@@ -2065,7 +2079,7 @@ def _dummy():
             self.__dict__.clear()
 
 
-    class DummyIO(_RunnableContext):
+    class DummyIO(util.RunnableContext):
 
         def __init__(self, stdin_content=None):
             self.stdin_content = stdin_content
@@ -2245,7 +2259,7 @@ def _dummy():
         def _wrap_func(self, func, block):
             tr = self
             def newfunc(*args, **kwargs):                # no 'self'
-                call = Call(None, _func_name(func), args, kwargs, None)
+                call = Call(None, util._func_name(func), args, kwargs, None)
                 tr.calls.append(call)
                 if block:
                     ret = block(func, *args, **kwargs)
@@ -2261,9 +2275,9 @@ def _dummy():
             func = method_obj
             tr = self
             def newfunc(self, *args, **kwargs):          # has 'self'
-                call = Call(self, _func_name(func), args, kwargs, None)
+                call = Call(self, util._func_name(func), args, kwargs, None)
                 tr.calls.append(call)
-                if _is_unbound(func): args = (self, ) + args   # call with 'self' if unbound method
+                if util._is_unbound(func): args = (self, ) + args   # call with 'self' if unbound method
                 if block:
                     ret = block(func, *args, **kwargs)
                 else:
@@ -2345,6 +2359,7 @@ del _dummy
 ##
 ## mainapp
 ##
+import unittest
 
 def load_module(mod_name, filepath, content=None):
     mod = type(os)(mod_name)
@@ -2428,14 +2443,13 @@ def _dummy():
             return modules
 
         def _load_classes(self, modules, pattern=None):
-            import unittest
             from fnmatch import fnmatch
             testclasses = []
             unittest_testclasses = []
             oktest_testclasses   = []
             for mod in modules:
                 for k in dir(mod):
-                    if k.startswith('_'): continue
+                    #if k.startswith('_'): continue
                     v = getattr(mod, k)
                     if not isinstance(v, type): continue
                     klass = v
@@ -2444,7 +2458,7 @@ def _dummy():
                     if issubclass(klass, unittest.TestCase):
                         testclasses.append(klass)
                         unittest_testclasses.append(klass)
-                    elif re.search(TARGET_PATTERN, klass.__name__):
+                    elif re.search(config.TARGET_PATTERN, klass.__name__):
                         testclasses.append(klass)
                         oktest_testclasses.append(klass)
             return testclasses, unittest_testclasses, oktest_testclasses
@@ -2452,8 +2466,6 @@ def _dummy():
         def _run_unittest(self, klasses, pattern=None, filters=None):
             self._trace("test_pattern: %r" % (pattern,))
             self._trace("unittest_testclasses: ", klasses)
-            import unittest
-            from fnmatch import fnmatch
             loader = unittest.TestLoader()
             the_suite = unittest.TestSuite()
             rexp = re.compile(r'^test(_|_\d\d\d(_|: ))?')
@@ -2471,7 +2483,8 @@ def _dummy():
                 else:
                     suite = loader.loadTestsFromTestCase(klass)
                 the_suite.addTest(suite)
-            runner = unittest.TextTestRunner()
+            #runner = unittest.TextTestRunner()
+            runner = unittest.TextTestRunner(stream=sys.stderr)
             result = runner.run(the_suite)
             n_errors = len(result.errors) + len(result.failures)
             return n_errors
@@ -2481,8 +2494,8 @@ def _dummy():
             self._trace("oktest_testclasses: ", klasses)
             if pattern:
                 kwargs.setdefault('filter', {})['test'] = pattern
-            import oktest
-            n_errors = oktest.run(*klasses, **kwargs)
+            import oktest; run = oktest.run    # don't remove!
+            n_errors = run(*klasses, **kwargs)
             return n_errors
 
         def _trace(self, msg, items=None):
@@ -2499,7 +2512,7 @@ def _dummy():
             buf = []; add = buf.append
             add("Usage: python -m oktest [options] file_or_directory...\n")
             #add(parser.help_message(20))
-            add(re.sub(r'^.*\n.*\nOptions:\n', '', parser.format_help()))
+            add(re.sub(r'^.*\n.*\n[oO]ptions:\n', '', parser.format_help()))
             add("Example:\n")
             add("   ## run test scripts in plain format\n")
             add("   $ python -m oktest -sp tests/*_test.py\n")
@@ -2533,7 +2546,6 @@ def _dummy():
             return filepaths
 
         def _find_files_recursively(self, testdir, pattern):
-            _trace = self._trace
             isdir = os.path.isdir
             assert isdir(testdir)
             filepaths = []
@@ -2541,7 +2553,7 @@ def _dummy():
                 files = rglob(testdir, pat)
                 if files:
                     filepaths.extend(files)
-                    _trace("testdir: %r, pattern: %r, files: " % (testdir, pat), files)
+                    self._trace("testdir: %r, pattern: %r, files: " % (testdir, pat), files)
             return filepaths
 
         #def _exclude_files(self, filepaths, pattern):
