@@ -1,5 +1,5 @@
 ###
-### $Release: 0.10.0 $
+### $Release: 0.11.0 $
 ### $Copyright: copyright(c) 2010-2011 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
@@ -250,6 +250,71 @@ attr('val'): 'aaa\nbbb\nccc\n' == 'aaa\nbbbb\nccc\n' : failed.
         else:
             raise AssertionError("AsertionError expected but not raised")
 
+    def test_matches(self):
+        # passed
+        try:
+            ok ("SOS").matches(r'^[A-Z]+$')
+        except:
+            raise
+        #
+        try:
+            ok ("SOS").matches(re.compile(r'^[A-Z]+$'))
+        except:
+            raise
+        #
+        try:
+            ok ("SOS").matches(r'^[A-Z]+$', re.M)
+        except:
+            raise
+        # failed
+        try:
+            ok ("SOS").matches(r'[0-9]+')
+        except AssertionError:
+            ex = sys.exc_info()[1]
+            self.assertEqual("re.search('[0-9]+', 'SOS') : failed.", str(ex))
+        else:
+            raise AssertionError("AssertionError expected but not raised")
+        #
+        try:
+            ok ("SOS").matches(re.compile(r'[0-9]+'))
+        except AssertionError:
+            ex = sys.exc_info()[1]
+            self.assertEqual("re.search('[0-9]+', 'SOS') : failed.", str(ex))
+        else:
+            raise AssertionError("AssertionError expected but not raised")
+
+    def test_not_match(self):
+        # passed
+        try:
+            ok ("SOS").not_match(r'^[0-9]+$')
+        except:
+            raise
+        #
+        try:
+            ok ("SOS").not_match(re.compile(r'^[0-9]+$'))
+        except:
+            raise
+        #
+        try:
+            ok ("SOS").not_match(r'^[0-9]+$', re.M)
+        except:
+            raise
+        # failed
+        try:
+            ok ("SOS").not_match(r'[A-Z]+')
+        except AssertionError:
+            ex = sys.exc_info()[1]
+            self.assertEqual("not re.search('[A-Z]+', 'SOS') : failed.", str(ex))
+        else:
+            raise AssertionError("AssertionError expected but not raised")
+        #
+        try:
+            ok ("SOS").not_match(re.compile(r'[A-Z]+'))
+        except AssertionError:
+            ex = sys.exc_info()[1]
+            self.assertEqual("not re.search('[A-Z]+', 'SOS') : failed.", str(ex))
+        else:
+            raise AssertionError("AssertionError expected but not raised")
 
     def test_raises(self):
         def f(): raise ValueError('errmsg1')
@@ -336,7 +401,17 @@ attr('val'): 'aaa\nbbb\nccc\n' == 'aaa\nbbbb\nccc\n' : failed.
             os.mkdir(dname)
             #
             ok (fname).is_file()
+            ok (dname).not_file()
             ok (dname).is_not_file()
+            #
+            @be_fail("os.path.isfile(%r) : failed." % dname)
+            def fn(): ok (dname).is_file()
+            @be_fail("os.path.isfile('not-exist') : failed.")
+            def fn(): ok ('not-exist').is_file()
+            @be_fail("not os.path.isfile(%r) : failed." % fname)
+            def fn(): ok (fname).not_file()
+            @be_fail("not os.path.isfile(%r) : failed." % fname)
+            def fn(): ok (fname).is_not_file()
         finally:
             os.unlink(fname)
             os.rmdir(dname)
@@ -352,7 +427,41 @@ attr('val'): 'aaa\nbbb\nccc\n' == 'aaa\nbbbb\nccc\n' : failed.
             os.mkdir(dname)
             #
             ok (dname).is_dir()
+            ok (fname).not_dir()
             ok (fname).is_not_dir()
+            #
+            @be_fail("os.path.isdir(%r) : failed." % fname)
+            def fn(): ok (fname).is_dir()
+            @be_fail("os.path.isdir('not-exist') : failed.")
+            def fn(): ok ('not-exist').is_dir()
+            @be_fail("not os.path.isdir(%r) : failed." % dname)
+            def fn(): ok (dname).not_dir()
+            @be_fail("not os.path.isdir(%r) : failed." % dname)
+            def fn(): ok (dname).is_not_dir()
+        finally:
+            os.unlink(fname)
+            os.rmdir(dname)
+
+
+    def test_exists(self):
+        fname = '__foobar.txt'
+        dname = '__foobar.d'
+        try:
+            f = open(fname, 'w')
+            f.write('foobar')
+            f.close()
+            os.mkdir(dname)
+            #
+            ok (dname).exists()
+            ok (fname).exists()
+            ok ("not-found").not_exist()
+            #
+            @be_fail("os.path.exists('not-exist') : failed.")
+            def fn(): ok ('not-exist').exists()
+            @be_fail("not os.path.exists(%r) : failed." % fname)
+            def fn(): ok (fname).not_exist()
+            @be_fail("not os.path.exists(%r) : failed." % dname)
+            def fn(): ok (dname).not_exist()
         finally:
             os.unlink(fname)
             os.rmdir(dname)
