@@ -1,5 +1,5 @@
 ###
-### $Release: 0.10.0 $
+### $Release: 0.11.0 $
 ### $Copyright: copyright(c) 2010-2011 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
@@ -9,7 +9,9 @@ import sys, os, re
 import unittest
 
 import oktest
-from oktest import ok, run, spec, Spec
+from oktest import ok, run
+from oktest import spec, Spec
+from oktest.util import Color
 os.environ['OKTEST_WARNING_DISABLED'] = 'true'
 
 with_stmt_available = sys.version_info >= (2, 5,)
@@ -25,6 +27,7 @@ def replace_with_stmt(content):
 class Spec_TC(unittest.TestCase):
 
     def test___init__(self):
+        """takes description."""
         assert Spec("SOS").desc == "SOS"
 
     def test___enter__(self):
@@ -77,10 +80,80 @@ class Spec_TC(unittest.TestCase):
         assert obj._stacktrace
         assert isinstance(obj._stacktrace, list)
 
-    def test_spec(self):
-        obj = spec("SOS")
-        assert isinstance(obj, Spec)
-        assert obj.desc == "SOS"
+    def test___iter___1(sefl):
+        """emurates with-stmt when used with for-stmt."""
+        called = []
+        def enter(*args):
+            called.append(('enter', args))
+        def exit(*args):
+            called.append(('exit', args))
+        obj = Spec('foo')
+        obj.__enter__ = enter
+        obj.__exit__  = exit
+        i = 0
+        for x in obj:
+            i += 1
+            called.append(('yield', x))
+            ok (x).is_(obj)
+        ok (i) == 1
+        ok (called[0]) == ('enter', ())
+        ok (called[1]) == ('yield', obj)
+        ok (called[2]) == ('exit', (None, None, None))
+
+    def test___call___1(self):
+        """emurates with-stmt when called as decorator."""
+        arr = []
+        def fn():
+            obj = Spec('foo')
+            @obj
+            def _():
+                arr.append("before")
+                ok (1+1) == 2     # passed
+                arr.append("after")
+        ok (fn).not_raise()
+        ok (arr) == ["before", "after"]
+
+    def test___call___2(self):
+        """emurates with-stmt when called as decorator."""
+        arr = []
+        def fn():
+            obj = Spec('foo')
+            @obj
+            def _():
+                arr.append("before")
+                ok (1+1) == 1    # failed
+                arr.append("after")
+        ok (fn).raises(AssertionError, "2 == 1 : failed.")
+        ok (arr) == ["before"]
+
+    def test___bool___1(self):
+        """returns True when $SPEC is not set."""
+        obj = Spec('SOS')
+        ok (bool(obj)) == True
+
+    def test___bool___2(self):
+        """returns False when $SPEC not matched to SPEC."""
+        obj = Spec('SOS')
+        try:
+            os.environ['SPEC'] = 'SOS'
+            ok (bool(obj)) == True
+            os.environ['SPEC'] = 'SOS!'
+            ok (bool(obj)) == False
+            os.environ['SPEC'] = 'OS'
+            ok (bool(obj)) == True
+        finally:
+            os.environ.pop('SPEC')
+
+
+    def test_spec_1(self):
+        """returns oktest.Spec object."""
+        obj = spec('Haruhi')
+        ok (obj).is_a(Spec)
+
+    def test_spec_2(self):
+        """takes description."""
+        obj = spec('Haruhi')
+        ok (obj.desc) == 'Haruhi'
 
 
 try:
@@ -174,8 +247,8 @@ AssertionError: 2 == 1 : failed.
 ----------------------------------------------------------------------
 ## total:1, passed:0, failed:1, error:0, skipped:0, todo:0  (0.000 sec)
 """[1:]
-        expected1 = oktest.Color._colorize(expected1)
-        expected2 = oktest.Color._colorize(expected2)
+        expected1 = Color._colorize(expected1)
+        expected2 = Color._colorize(expected2)
         fname = "__test_scenario1.py"
         ## with with-stmt
         if with_stmt_available:
@@ -258,8 +331,8 @@ AssertionError: 2 == 0 : failed.
 ----------------------------------------------------------------------
 ## total:1, passed:0, failed:1, error:0, skipped:0, todo:0  (0.000 sec)
 """[1:]
-        expected1 = oktest.Color._colorize(expected1)
-        expected2 = oktest.Color._colorize(expected2)
+        expected1 = Color._colorize(expected1)
+        expected2 = Color._colorize(expected2)
         fname = "__test_scenario2.py"
         ## with with-statement
         if with_stmt_available:
@@ -272,6 +345,8 @@ AssertionError: 2 == 0 : failed.
             input2 = replace_with_stmt(input)
             sout, serr = self._run_oktest(fname, input2)
             sout = re.sub(r'0.\d\d\d sec', '0.000 sec', sout)
+            ok (sout) == expected2
+            ok (serr) == ""
             assert sout == expected2
             assert serr == ""
 
@@ -311,8 +386,8 @@ AttributeError: 'NoneType' object has no attribute 'unknownattribute'
 ## total:1, passed:0, failed:0, error:1, skipped:0, todo:0  (0.000 sec)
 """[1:]
         expected2 = expected1
-        expected1 = oktest.Color._colorize(expected1)
-        expected2 = oktest.Color._colorize(expected2)
+        expected1 = Color._colorize(expected1)
+        expected2 = Color._colorize(expected2)
         fname = "__test_scenario3.py"
         ## with with-statement
         if with_stmt_available:
