@@ -1,5 +1,5 @@
 ###
-### $Release: 0.10.0 $
+### $Release: 0.11.0 $
 ### $Copyright: copyright(c) 2010-2011 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
@@ -9,7 +9,7 @@ import sys, os, re
 import unittest
 
 import oktest
-from oktest import ok, NG, run, spec
+from oktest import ok, NG, run
 from oktest.util import *
 from oktest.tracer import Tracer, Call, FakeObject
 
@@ -20,6 +20,9 @@ except ImportError:
 
 os.environ['OKTEST_WARNING_DISABLED'] = 'true'
 
+def spec(desc):
+    return bool(True)
+
 
 class Call_TC(unittest.TestCase):
 
